@@ -4,16 +4,11 @@
 import urlparse
 
 
-class Facter(object):
-    unit = 'value'
+class Baser(object):
 
     def __init__(self, reviewer):
         self.reviewer = reviewer
 
-    @classmethod
-    def get_fact_definitions(cls):
-        raise NotImplementedError
-
     @property
     def page_uuid(self):
         return self.reviewer.page_uuid
@@ -30,17 +25,11 @@ class Facter(object):
     def config(self):
         return self.reviewer.config
 
-    def get(self, url):
-        return self.reviewer._get(url)
+    def is_absolute(self, url):
+        return bool(urlparse.urlparse(url).scheme)
 
-    def add_fact(self, key, value):
-        self.reviewer.add_fact(key, value)
-
-    def add_violation(self, key, value):
-        self.reviewer.add_violation(key, value)
-
-    def async_get(self, url, handler, method='GET', **kw):
-        self.reviewer._async_get(url, handler, method, **kw)
+    def rebase(self, url):
+        return urlparse.urljoin(self.page_url, url)
 
     def is_valid(self, url):
         try:
@@ -48,11 +37,26 @@ class Facter(object):
         except ValueError:
             return None
 
-    def is_absolute(self, url):
-        return bool(urlparse.urlparse(url).scheme)
-
-    def rebase(self, url):
-        return urlparse.urljoin(self.page_url.rstrip('/'), url)
-
     def to_gzip(self, content):
         return content.encode('zip')
+
+    def add_fact(self, key, value):
+        self.reviewer.add_fact(key, value)
+
+
+class Facter(Baser):
+
+    unit = 'value'
+
+    @classmethod
+    def get_fact_definitions(cls):
+        raise NotImplementedError
+
+    def add_violation(self, key, value):
+        self.reviewer.add_violation(key, value)
+
+    def get(self, url):
+        return self.reviewer._get(url)
+
+    def async_get(self, url, handler, method='GET', **kw):
+        self.reviewer._async_get(url, handler, method, **kw)
