@@ -11,7 +11,6 @@ import logging
 import sqlalchemy as sa
 from sqlalchemy.orm import relationship
 from sqlalchemy import or_
-from sqlalchemy.exc import OperationalError
 from ujson import dumps
 from tornado.concurrent import return_future
 
@@ -120,7 +119,7 @@ class Page(Base):
                     raise
 
     @classmethod
-    def get_next_job(cls, db, expiration):
+    def get_next_job(cls, db, expiration, cache, lock_expiration):
         from holmes.models import Settings, Domain
 
         expired_time = datetime.now() - timedelta(seconds=expiration)
@@ -156,10 +155,21 @@ class Page(Base):
 
         page = choice(pages_in_need_of_review)
 
+        for i in range(10):
+            lock = cache.has_next_job_lock(page.url, lock_expiration)
+            if lock is None:
+                page = choice(pages_in_need_of_review)
+            else:
+                break
+
+        if lock is None:
+            return None
+
         return {
             'page': str(page.uuid),
             'url': page.url,
-            'score': page.score
+            'score': page.score,
+            'lock': lock
         }
 
     @classmethod
@@ -231,11 +241,8 @@ class Page(Base):
 
     @classmethod
     def insert_or_update_page(cls, url, score, domain, db, publish_method):
-        page = db.query(Page).filter(or_(
-            Page.url == url,
-            Page.url == url.rstrip('/'),
-            Page.url == "%s/" % url
-        )).first()
+        url_hash = hashlib.sha512(url).hexdigest()
+        page = db.query(Page).filter(Page.url_hash==url_hash).first()
 
         if page:
             for i in range(3):
@@ -255,8 +262,6 @@ class Page(Base):
 
             return page.uuid
 
-        url_hash = hashlib.sha512(url).hexdigest()
-
         db.begin(subtransactions=True)
         try:
             page = Page(url=url, url_hash=url_hash, domain=domain, score=score)
@@ -264,11 +269,11 @@ class Page(Base):
             db.flush()
             db.commit()
         except Exception:
+            db.rollback()
             err = sys.exc_info()[1]
             if 'Duplicate entry' in str(err):
                 logging.error('Duplicate entry! (Details: %s)' % str(err))
             else:
-                db.rollback()
                 raise
 
         publish_method(dumps({
